"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignupController = void 0;
const tslib_1 = require("tslib");
// Uncomment these imports to begin using these cool features!
const jwt = tslib_1.__importStar(require("jsonwebtoken"));
const core_1 = require("@loopback/core");
const rest_1 = require("@loopback/rest");
const authentication_1 = require("@bleco/authentication");
const authorization_1 = require("@bleco/authorization");
const core_2 = require("@loopx/core");
const models_1 = require("../models");
const local_user_profile_1 = require("../models/local-user-profile");
const signup_fast_request_dto_model_1 = require("../models/signup-fast-request.dto.model");
const signup_request_dto_model_1 = require("../models/signup-request-dto.model");
const signup_request_model_1 = require("../models/signup-request.model");
const signup_with_token_response_dto_model_1 = require("../models/signup-with-token-response-dto.model");
const auth_1 = require("../modules/auth");
const providers_1 = require("../providers");
const services_1 = require("../services");
const successResponse = 'Success Response.';
const basePath = '/auth/signup';
let SignupController = exports.SignupController = class SignupController {
    constructor(preSignupFn, userSignupFn, signupHelperService, tokenService, getAuthCode) {
        this.preSignupFn = preSignupFn;
        this.userSignupFn = userSignupFn;
        this.signupHelperService = signupHelperService;
        this.tokenService = tokenService;
        this.getAuthCode = getAuthCode;
    }
    async signupFast(signupFastRequest, client) {
        await this.signupHelperService.validateSignupFastRequest(signupFastRequest);
        const user = new auth_1.AuthUser(await this.userSignupFn(signupFastRequest, {}));
        user.permissions = [];
        const token = await this.getAuthCode(client, user);
        return {
            code: token,
        };
    }
    async requestSignup(signupRequest, client, handler) {
        var _a;
        // Default expiry is 30 minutes
        const expiryDuration = parseInt((_a = process.env.REQUEST_SIGNUP_LINK_EXPIRY) !== null && _a !== void 0 ? _a : '1800');
        const codePayload = await this.preSignupFn(signupRequest);
        // TODO: Add client id in preSignupFn?
        codePayload.clientId = client.clientId;
        const token = jwt.sign(codePayload, process.env.JWT_SECRET, {
            expiresIn: expiryDuration,
            subject: signupRequest.email,
            issuer: process.env.JWT_ISSUER,
            algorithm: 'HS256',
        });
        await handler({
            code: token,
            expiry: expiryDuration,
            email: signupRequest.email,
        });
    }
    async signupWithToken(req, signupUser) {
        const user = await this.userSignupFn(req, signupUser);
        return new signup_with_token_response_dto_model_1.SignupWithTokenResponseDto({
            email: req.email,
            user,
        });
    }
    async verifyInviteToken(signupUser) {
        return signupUser;
    }
};
tslib_1.__decorate([
    (0, authentication_1.authenticateClient)(authentication_1.STRATEGY.CLIENT_PASSWORD),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)(`${basePath}/fast`, {
        responses: {
            [core_2.STATUS_CODE.NO_CONTENT]: {
                description: successResponse,
            },
            ...core_2.ErrorCodes,
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)()),
    tslib_1.__param(1, (0, core_1.inject)(authentication_1.AuthenticationBindings.CURRENT_CLIENT)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [signup_fast_request_dto_model_1.SignupFastRequestDto,
        models_1.AuthClient]),
    tslib_1.__metadata("design:returntype", Promise)
], SignupController.prototype, "signupFast", null);
tslib_1.__decorate([
    (0, authentication_1.authenticateClient)(authentication_1.STRATEGY.CLIENT_PASSWORD),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)(`${basePath}/create-token`, {
        responses: {
            [core_2.STATUS_CODE.NO_CONTENT]: {
                description: successResponse,
            },
            ...core_2.ErrorCodes,
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)()),
    tslib_1.__param(1, (0, core_1.inject)(authentication_1.AuthenticationBindings.CURRENT_CLIENT)),
    tslib_1.__param(2, (0, core_1.inject)(providers_1.SignUpBindings.SIGNUP_HANDLER_PROVIDER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [signup_request_dto_model_1.SignupRequestDto,
        models_1.AuthClient, Function]),
    tslib_1.__metadata("design:returntype", Promise)
], SignupController.prototype, "requestSignup", null);
tslib_1.__decorate([
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.BEARER, {}, undefined, providers_1.VerifyBindings.BEARER_SIGNUP_VERIFY_PROVIDER),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)(`${basePath}/create-user`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [core_2.STATUS_CODE.OK]: {
                description: successResponse,
                content: {
                    [core_2.CONTENT_TYPE.JSON]: {
                        schema: (0, rest_1.getModelSchemaRef)(local_user_profile_1.LocalUserEmailPasswordProfileDto),
                    },
                },
            },
            ...core_2.ErrorCodes,
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)()),
    tslib_1.__param(1, (0, core_1.inject)(authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [local_user_profile_1.LocalUserEmailPasswordProfileDto,
        signup_request_model_1.SignupRequest]),
    tslib_1.__metadata("design:returntype", Promise)
], SignupController.prototype, "signupWithToken", null);
tslib_1.__decorate([
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.BEARER, {}, undefined, providers_1.VerifyBindings.BEARER_SIGNUP_VERIFY_PROVIDER),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.get)(`${basePath}/verify-token`, {
        security: core_2.OPERATION_SECURITY_SPEC,
        responses: {
            [core_2.STATUS_CODE.OK]: {
                description: successResponse,
            },
            ...core_2.ErrorCodes,
        },
    }),
    tslib_1.__param(0, (0, core_1.inject)(authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [signup_request_model_1.SignupRequest]),
    tslib_1.__metadata("design:returntype", Promise)
], SignupController.prototype, "verifyInviteToken", null);
exports.SignupController = SignupController = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(providers_1.SignUpBindings.PRE_LOCAL_SIGNUP_PROVIDER)),
    tslib_1.__param(1, (0, core_1.inject)(providers_1.SignUpBindings.LOCAL_SIGNUP_PROVIDER)),
    tslib_1.__param(2, (0, core_1.service)(services_1.SignupHelperService)),
    tslib_1.__param(3, (0, core_1.service)(services_1.TokenService)),
    tslib_1.__param(4, (0, core_1.inject)(providers_1.AuthCodeBindings.AUTH_CODE_GENERATOR_PROVIDER)),
    tslib_1.__metadata("design:paramtypes", [Function, Function, services_1.SignupHelperService,
        services_1.TokenService, Function])
], SignupController);
//# sourceMappingURL=signup.controller.js.map