"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthaLoginController = void 0;
const tslib_1 = require("tslib");
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const authentication_1 = require("@bleco/authentication");
const authorization_1 = require("@bleco/authorization");
const core_1 = require("@loopx/core");
const providers_1 = require("../../providers");
const repositories_1 = require("../../repositories");
const client_auth_request_dto_1 = require("./models/client-auth-request.dto");
const token_response_dto_1 = require("./models/token-response.dto");
const AuthSessionKey = 'autha';
const LoginSessionKey = 'login';
let AuthaLoginController = exports.AuthaLoginController = class AuthaLoginController {
    constructor(authClientRepository, logger, getAuthCode) {
        this.authClientRepository = authClientRepository;
        this.logger = logger;
        this.getAuthCode = getAuthCode;
    }
    // TODO
    async loginViaAutha(clientId, clientChallenge, clientChallengeMethod) {
        //do nothing
    }
    async postLoginViaAutha(clientCreds) {
        //do nothing
    }
    async authaCallback(code, request, response, user) {
        const { clientId } = request.session[LoginSessionKey];
        delete request.session[LoginSessionKey];
        if (!clientId || !user) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        const client = await this.authClientRepository.findOne({
            where: {
                clientId,
            },
        });
        if (!(client === null || client === void 0 ? void 0 : client.redirectUrl)) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        try {
            const token = await this.getAuthCode(client, user);
            response.redirect(`${client.redirectUrl}?code=${token}`);
        }
        catch (error) {
            this.logger.error(error);
            throw new authentication_1.AuthenticationErrors.InvalidCredentials();
        }
    }
};
tslib_1.__decorate([
    (0, authentication_1.authenticateClient)(authentication_1.STRATEGY.CLIENT_PASSWORD),
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.AUTHA, {
        sessionKey: AuthSessionKey,
    }, req => {
        var _a;
        // set the client id in the session for later use temporarily
        req.session[LoginSessionKey] = { clientId: req.body.client_id };
        return {
            interactionMode: (_a = req.query.interaction_mode) !== null && _a !== void 0 ? _a : req.query.interactionMode,
        };
    }),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.get)('/auth/autha', {
        responses: {
            [core_1.STATUS_CODE.PERMANENT_REDIRECT]: {
                description: 'Redirect to Autha login page',
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.query.string('client_id')),
    tslib_1.__param(1, rest_1.param.query.string('client_challenge')),
    tslib_1.__param(2, rest_1.param.query.string('client_challenge_method')),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String, String]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthaLoginController.prototype, "loginViaAutha", null);
tslib_1.__decorate([
    (0, authentication_1.authenticateClient)(authentication_1.STRATEGY.CLIENT_PASSWORD),
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.AUTHA, {
        sessionKey: AuthSessionKey,
    }, req => {
        var _a;
        // set the client id in the session for later use temporarily
        req.session[LoginSessionKey] = { clientId: req.body.client_id };
        return {
            interactionMode: (_a = req.query.interaction_mode) !== null && _a !== void 0 ? _a : req.query.interactionMode,
        };
    }),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)('/auth/autha', {
        responses: {
            [core_1.STATUS_CODE.OK]: {
                description: 'POST Call for Autha based login',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { [core_1.X_TS_TYPE]: token_response_dto_1.TokenResponse },
                    },
                },
            },
        },
    }),
    tslib_1.__param(0, (0, rest_1.requestBody)({
        content: {
            [core_1.CONTENT_TYPE.FORM_URLENCODED]: {
                schema: (0, rest_1.getModelSchemaRef)(client_auth_request_dto_1.ClientAuthRequest),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [client_auth_request_dto_1.ClientAuthRequest]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthaLoginController.prototype, "postLoginViaAutha", null);
tslib_1.__decorate([
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.AUTHA, {
        sessionKey: AuthSessionKey,
    }, req => ({})),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.get)('/auth/autha-redirect', {
        responses: {
            [core_1.STATUS_CODE.OK]: {
                description: 'Autha Redirect Token Response',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { [core_1.X_TS_TYPE]: token_response_dto_1.TokenResponse },
                    },
                },
            },
        },
    }),
    tslib_1.__param(0, rest_1.param.query.string('code')),
    tslib_1.__param(1, (0, context_1.inject)(rest_1.RestBindings.Http.REQUEST)),
    tslib_1.__param(2, (0, context_1.inject)(rest_1.RestBindings.Http.RESPONSE)),
    tslib_1.__param(3, (0, context_1.inject)(authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object, Object, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthaLoginController.prototype, "authaCallback", null);
exports.AuthaLoginController = AuthaLoginController = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(repositories_1.AuthClientRepository)),
    tslib_1.__param(1, (0, context_1.inject)(core_1.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(2, (0, context_1.inject)(providers_1.AuthCodeBindings.AUTH_CODE_GENERATOR_PROVIDER)),
    tslib_1.__metadata("design:paramtypes", [repositories_1.AuthClientRepository, Object, Function])
], AuthaLoginController);
//# sourceMappingURL=autha-login.controller.js.map