"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogoutController = void 0;
const tslib_1 = require("tslib");
const crypto_1 = tslib_1.__importDefault(require("crypto"));
const https_proxy_agent_1 = require("https-proxy-agent");
const url_1 = require("url");
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const authentication_1 = require("@bleco/authentication");
const authorization_1 = require("@bleco/authorization");
const core_1 = require("@loopx/core");
const keys_1 = require("../../keys");
const models_1 = require("../../models");
const repositories_1 = require("../../repositories");
const keys_2 = require("./keys");
// eslint-disable-next-line @typescript-eslint/no-unused-vars
const LogoutSessionKey = 'logout';
const proxyUrl = (_a = process.env.HTTPS_PROXY) !== null && _a !== void 0 ? _a : process.env.HTTP_PROXY;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
const getProxyAgent = () => {
    if (proxyUrl) {
        return new https_proxy_agent_1.HttpsProxyAgent(proxyUrl);
    }
    return undefined;
};
const size = 16;
let LogoutController = exports.LogoutController = class LogoutController {
    constructor(ctx, res, revokedTokens, refreshTokenRepo, userRepo, authClientRepo, logger, actorKey, userTenantRepo, loginActivityRepo, userActivity) {
        this.ctx = ctx;
        this.res = res;
        this.revokedTokens = revokedTokens;
        this.refreshTokenRepo = refreshTokenRepo;
        this.userRepo = userRepo;
        this.authClientRepo = authClientRepo;
        this.logger = logger;
        this.actorKey = actorKey;
        this.userTenantRepo = userTenantRepo;
        this.loginActivityRepo = loginActivityRepo;
        this.userActivity = userActivity;
    }
    async logout(auth, req, currentUser) {
        var _a;
        const token = auth === null || auth === void 0 ? void 0 : auth.replace(/bearer /i, '');
        if (!token || !req.refreshToken) {
            throw new core_1.AuthErrors.TokenMissing();
        }
        const refreshTokenModel = await this.refreshTokenRepo.get(req.refreshToken);
        if (!refreshTokenModel) {
            throw new authentication_1.AuthenticationErrors.TokenExpired();
        }
        let logoutUrl;
        if (refreshTokenModel.externalRefreshToken) {
            const creds = await this.userRepo.credentials(refreshTokenModel.userId).get();
            try {
                const logoutFn = await this.ctx.get((0, keys_2.buildLogoutBindingKey)(creds.authProvider), {
                    optional: true,
                });
                logoutUrl = await (logoutFn === null || logoutFn === void 0 ? void 0 : logoutFn(refreshTokenModel.externalRefreshToken, `client_id=${refreshTokenModel.clientId}`));
                this.logger.info(`User ${refreshTokenModel.username} logged off successfully from ${creds.authProvider} and return logout url ${logoutUrl !== null && logoutUrl !== void 0 ? logoutUrl : '<empty>'}`);
            }
            catch (e) {
                this.logger.error(`Error while logging off from ${creds.authProvider}. Error :: ${e} ${JSON.stringify(e)}`);
            }
        }
        if (refreshTokenModel.accessToken !== token) {
            throw new authentication_1.AuthenticationErrors.TokenInvalid();
        }
        await this.revokedTokens.set(token, { token });
        await this.refreshTokenRepo.delete(req.refreshToken);
        if (refreshTokenModel.pubnubToken) {
            await this.refreshTokenRepo.delete(refreshTokenModel.pubnubToken);
        }
        const user = await this.userRepo.findById(refreshTokenModel.userId);
        const userTenant = await this.userTenantRepo.findOne({
            where: { userId: user.id },
        });
        if ((_a = this.userActivity) === null || _a === void 0 ? void 0 : _a.markUserActivity)
            this.markUserActivity(refreshTokenModel, user, userTenant);
        return new core_1.SuccessResponse({
            success: true,
            key: refreshTokenModel.userId,
            logoutUrl,
        });
    }
    async logoutRedirect(state) {
        const clientId = new url_1.URLSearchParams(state).get('client_id');
        if (!clientId) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        const authClient = await this.authClientRepo.findOne({ where: { clientId } });
        if (!authClient) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid();
        }
        if (!authClient.logoutRedirectUrl) {
            throw new authentication_1.AuthenticationErrors.ClientInvalid('logout_redirect_url is not configured');
        }
        this.res.redirect(authClient.logoutRedirectUrl);
    }
    markUserActivity(refreshTokenModel, user, userTenant) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        const encryptionKey = process.env.ENCRYPTION_KEY;
        if (encryptionKey) {
            const iv = crypto_1.default.randomBytes(size);
            /* encryption of IP Address */
            const cipherIp = crypto_1.default.createCipheriv('aes-256-gcm', encryptionKey, iv);
            const ip = (_d = (_b = (_a = this.ctx.request.headers['x-forwarded-for']) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : (_c = this.ctx.request.socket.remoteAddress) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : '';
            const encryptIp = Buffer.concat([cipherIp.update(ip, 'utf8'), cipherIp.final()]);
            const authTagIp = cipherIp.getAuthTag();
            const ipAddress = JSON.stringify({
                iv: iv.toString('hex'),
                encryptedData: encryptIp.toString('hex'),
                authTag: authTagIp.toString('hex'),
            });
            /* encryption of Paylolad Address */
            const cipherPayload = crypto_1.default.createCipheriv('aes-256-gcm', encryptionKey, iv);
            const activityPayload = JSON.stringify({
                ...user,
                clientId: refreshTokenModel.clientId,
            });
            const encryptPayload = Buffer.concat([cipherPayload.update(activityPayload, 'utf8'), cipherPayload.final()]);
            const authTagPayload = cipherIp.getAuthTag();
            const tokenPayload = JSON.stringify({
                iv: iv.toString('hex'),
                encryptedData: encryptPayload.toString('hex'),
                authTag: authTagPayload.toString('hex'),
            });
            let actor, tenantId;
            if (userTenant) {
                actor = (_f = (_e = userTenant[this.actorKey]) === null || _e === void 0 ? void 0 : _e.toString()) !== null && _f !== void 0 ? _f : '0';
                tenantId = userTenant.tenantId;
            }
            else {
                actor = (_h = (_g = user['id']) === null || _g === void 0 ? void 0 : _g.toString()) !== null && _h !== void 0 ? _h : '0';
                tenantId = (_j = user.defaultTenantId) !== null && _j !== void 0 ? _j : '0';
            }
            const loginActivity = new models_1.LoginActivity({
                actor,
                tenantId,
                loginTime: new Date(),
                tokenPayload,
                deviceInfo: (_k = this.ctx.request.headers['user-agent']) === null || _k === void 0 ? void 0 : _k.toString(),
                loginType: "LOGOUT" /* LoginType.LOGOUT */,
                ipAddress,
            });
            this.loginActivityRepo.create(loginActivity).catch(() => {
                this.logger.error(`Failed to add the login activity => ${JSON.stringify(loginActivity)}`);
            });
        }
    }
};
tslib_1.__decorate([
    (0, authentication_1.authenticate)(authentication_1.STRATEGY.BEARER, {
        passReqToCallback: true,
    }),
    (0, authorization_1.authorize)({ permissions: ['*'] }),
    (0, rest_1.post)('/logout', {
        security: core_1.OPERATION_SECURITY_SPEC,
        description: 'To logout',
        responses: {
            [core_1.STATUS_CODE.OK]: {
                description: 'Success Response',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { [core_1.X_TS_TYPE]: core_1.SuccessResponse },
                    },
                },
            },
            ...core_1.ErrorCodes,
        },
    }),
    tslib_1.__param(0, rest_1.param.header.string('Authorization', {
        description: 'This is the access token which is required to authenticate user.',
    })),
    tslib_1.__param(1, (0, rest_1.requestBody)({
        content: {
            [core_1.CONTENT_TYPE.JSON]: {
                schema: (0, rest_1.getModelSchemaRef)(models_1.RefreshTokenRequest, {
                    partial: true,
                }),
            },
        },
    })),
    tslib_1.__param(2, (0, context_1.inject)(authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, models_1.RefreshTokenRequest, Object]),
    tslib_1.__metadata("design:returntype", Promise)
], LogoutController.prototype, "logout", null);
tslib_1.__decorate([
    (0, rest_1.get)('/logout/redirect'),
    tslib_1.__param(0, rest_1.param.query.string('state')),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", Promise)
], LogoutController.prototype, "logoutRedirect", null);
exports.LogoutController = LogoutController = tslib_1.__decorate([
    tslib_1.__param(0, context_1.inject.context()),
    tslib_1.__param(1, (0, context_1.inject)(rest_1.RestBindings.Http.RESPONSE)),
    tslib_1.__param(2, (0, repository_1.repository)(repositories_1.RevokedTokenRepository)),
    tslib_1.__param(3, (0, repository_1.repository)(repositories_1.RefreshTokenRepository)),
    tslib_1.__param(4, (0, repository_1.repository)(repositories_1.UserRepository)),
    tslib_1.__param(5, (0, repository_1.repository)(repositories_1.AuthSecureClientRepository)),
    tslib_1.__param(6, (0, context_1.inject)(core_1.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(7, (0, context_1.inject)(keys_1.AuthServiceBindings.ActorIdKey)),
    tslib_1.__param(8, (0, repository_1.repository)(repositories_1.UserTenantRepository)),
    tslib_1.__param(9, (0, repository_1.repository)(repositories_1.LoginActivityRepository)),
    tslib_1.__param(10, (0, context_1.inject)(keys_1.AuthServiceBindings.MarkUserActivity, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [rest_1.RequestContext, Object, repositories_1.RevokedTokenRepository,
        repositories_1.RefreshTokenRepository,
        repositories_1.UserRepository,
        repositories_1.AuthSecureClientRepository, Object, String, repositories_1.UserTenantRepository,
        repositories_1.LoginActivityRepository, Object])
], LogoutController);
//# sourceMappingURL=logout.controller.js.map