"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthCodeGeneratorProvider = void 0;
const tslib_1 = require("tslib");
const jwt = tslib_1.__importStar(require("jsonwebtoken"));
const context_1 = require("@loopback/context");
const core_1 = require("@loopback/core");
const authentication_1 = require("@bleco/authentication");
const enums_1 = require("../enums");
const keys_1 = require("../keys");
const otp_1 = require("../otp/otp");
const services_1 = require("../services");
const keys_2 = require("./keys");
let AuthCodeGeneratorProvider = exports.AuthCodeGeneratorProvider = class AuthCodeGeneratorProvider {
    constructor(otpService, checkMfa, codeWriter, mfaConfig, otpConfig) {
        this.otpService = otpService;
        this.checkMfa = checkMfa;
        this.codeWriter = codeWriter;
        this.mfaConfig = mfaConfig;
        this.otpConfig = otpConfig;
    }
    value() {
        return async (client, user) => {
            const codePayload = {
                clientId: client.clientId,
                user,
            };
            const isMfaEnabled = await this.checkMfa(user);
            if (isMfaEnabled) {
                codePayload.mfa = true;
                if (this.mfaConfig.secondFactor === authentication_1.STRATEGY.OTP && this.otpConfig.method === enums_1.OtpMethodType.OTP) {
                    await this.otpService.sendOtp(otp_1.OtpRequest.fromUser(user, user.username), client);
                }
            }
            return this.codeWriter(jwt.sign(codePayload, client.secret, {
                expiresIn: client.authCodeExpiration,
                audience: client.clientId,
                issuer: process.env.JWT_ISSUER,
                algorithm: 'HS256',
            }));
        };
    }
};
exports.AuthCodeGeneratorProvider = AuthCodeGeneratorProvider = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.service)(services_1.OtpService)),
    tslib_1.__param(1, (0, context_1.inject)(keys_2.VerifyBindings.MFA_PROVIDER)),
    tslib_1.__param(2, (0, context_1.inject)(keys_2.AuthCodeBindings.CODEWRITER_PROVIDER)),
    tslib_1.__param(3, (0, context_1.inject)(keys_1.AuthServiceBindings.MfaConfig, { optional: true })),
    tslib_1.__param(4, (0, context_1.inject)(keys_1.AuthServiceBindings.OtpConfig, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [services_1.OtpService, Function, Function, Object, Object])
], AuthCodeGeneratorProvider);
//# sourceMappingURL=auth-code-generator.provider.js.map