"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserTenantRepository = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const soft_delete_1 = require("@bleco/soft-delete");
const models_1 = require("../models");
const types_1 = require("../types");
let UserTenantRepository = exports.UserTenantRepository = class UserTenantRepository extends soft_delete_1.DefaultSoftCrudRepository {
    constructor(dataSource, tenantRepositoryGetter, userRepositoryGetter, roleRepositoryGetter, userLevelPermissionRepositoryGetter, entityClass = models_1.UserTenant) {
        super(entityClass, dataSource);
        this.tenantRepositoryGetter = tenantRepositoryGetter;
        this.userRepositoryGetter = userRepositoryGetter;
        this.roleRepositoryGetter = roleRepositoryGetter;
        this.userLevelPermissionRepositoryGetter = userLevelPermissionRepositoryGetter;
        this.userLevelPermissions = this.createHasManyRepositoryFactoryFor('userLevelPermissions', userLevelPermissionRepositoryGetter);
        this.registerInclusionResolver('userLevelPermissions', this.userLevelPermissions.inclusionResolver);
        this.role = this.createBelongsToAccessorFor('role', roleRepositoryGetter);
        this.registerInclusionResolver('role', this.role.inclusionResolver);
        this.user = this.createBelongsToAccessorFor('user', userRepositoryGetter);
        this.registerInclusionResolver('user', this.user.inclusionResolver);
        this.tenant = this.createBelongsToAccessorFor('tenant', tenantRepositoryGetter);
        this.registerInclusionResolver('tenant', this.tenant.inclusionResolver);
    }
};
exports.UserTenantRepository = UserTenantRepository = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(`datasources.${types_1.AuthDbSourceName}`)),
    tslib_1.__param(1, repository_1.repository.getter('TenantRepository')),
    tslib_1.__param(2, repository_1.repository.getter('UserRepository')),
    tslib_1.__param(3, repository_1.repository.getter('RoleRepository')),
    tslib_1.__param(4, repository_1.repository.getter('UserLevelPermissionRepository')),
    tslib_1.__metadata("design:paramtypes", [repository_1.juggler.DataSource, Function, Function, Function, Function, Object])
], UserTenantRepository);
//# sourceMappingURL=user-tenant.repository.js.map